@@ -1,5 +1,5 @@
 /*
- * $Id$
+ * $Id: abook.c,v 1.36 2002/10/11 11:04:15 jheinonen Exp $
  *
  * by JH <jheinonen@users.sourceforge.net>
  *
@@ -12,6 +12,7 @@
 #include <sys/stat.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <errno.h>
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
 #endif
@@ -27,15 +28,16 @@
 #include "misc.h"
 #include "options.h"
 #include "getname.h"
+#include "getopt.h"
+#include <assert.h>
 
 static void             init_abook();
+static void		quit_abook_sig(int i);
 static void             set_filenames();
-static void		free_filenames();
 static void             parse_command_line(int argc, char **argv);
 static void             show_usage();
 static void             mutt_query(char *str);
 static void             init_mutt_query();
-static void             quit_mutt_query();
 static void		convert(char *srcformat, char *srcfile,
 				char *dstformat, char *dstfile);
 static void		add_email(int);
@@ -43,40 +45,93 @@ static void		add_email(int);
 char *datafile = NULL;
 char *rcfile = NULL;
 
+bool alternative_datafile = FALSE;
+bool alternative_rcfile = FALSE;
+
+static int
+datafile_writeable()
+{
+	FILE *f;
+
+	assert(datafile != NULL);
+
+	if( (f = fopen(datafile, "a")) == NULL)
+		return FALSE;
+
+	fclose(f);
+
+	return TRUE;
+}
+
+static void
+check_abook_directory()
+{
+	struct stat s;
+	char *dir;
+
+	assert(!is_ui_initialized());
+
+	if(alternative_datafile)
+		return;
+
+	dir = strconcat(getenv("HOME"), "/" DIR_IN_HOME, NULL);
+	assert(dir != NULL);
+
+	if(stat(dir, &s) == -1) {
+		if(errno != ENOENT) {
+			perror(dir);
+                        free(dir);
+                        exit(1);
+		}
+		if(mkdir(dir, 0700) == -1) {
+			printf("Cannot create directory %s\n", dir);
+			perror(dir);
+			free(dir);
+			exit(1);
+		}
+	} else if(!S_ISDIR(s.st_mode)) {
+		printf("%s is not a directory\n", dir);
+		free(dir);
+		exit(1);
+	}
+
+	free(dir);
+}
+
 static void
 init_abook()
 {
 	set_filenames();
-	init_options();
+	check_abook_directory();
+	init_opts();
+	if(load_opts(rcfile) > 0) {
+		printf("Press enter to continue...\n");
+		fgetc(stdin);
+	}
+
+	signal(SIGKILL, quit_abook_sig);
+	signal(SIGTERM, quit_abook_sig);
 
-	signal(SIGKILL, quit_abook);
-	signal(SIGTERM, quit_abook);
-	
 	if( init_ui() )
 		exit(1);
-	
+
 	umask(DEFAULT_UMASK);
 
-	/*
-	 * this is very ugly for now
-	 */
-	/*if( options_get_int("datafile", "autosave") )*/
-
-	if( load_database(datafile) == 2 ) {
-		char *tmp = strconcat(getenv("HOME"),
-				"/" DATAFILE, NULL);
-
-		if( safe_strcmp(tmp, datafile) ) {
-			refresh_screen();
-			statusline_msg("Sorry, the specified file does "
-				"not appear to be a valid abook addressbook");
-			statusline_msg("Will open default addressbook...");
-			free(datafile);
-			datafile = tmp;
-			load_database(datafile);
-		} else
-			free(tmp);
-	}
+	if(!datafile_writeable()) {
+		char *s = mkstr("File %s is not writeable", datafile);
+		refresh_screen();
+		statusline_msg(s);
+		free(s);
+		if(load_database(datafile) || !statusline_ask_boolean(
+					"If you continue all changes will "
+				"be lost. Do you want to continue?", FALSE)) {
+			free_opts();
+			/*close_database();*/
+			close_ui();
+			exit(1);
+		}
+	} else
+		load_database(datafile);
 
 	refresh_screen();
 }
@@ -84,37 +139,51 @@ init_abook()
 void
 quit_abook()
 {
-	if( options_get_int("autosave") )
+	if( opt_get_bool(BOOL_AUTOSAVE) )
 		save_database();
 	else if( statusline_ask_boolean("Save database", TRUE) )
 		save_database();
 
-	close_config();
+	free_opts();
 	close_database();
 
 	close_ui();
-	
+
 	exit(0);
 }
 
+static void
+quit_abook_sig(int i)
+{
+	quit_abook();
+}
+
 int
 main(int argc, char **argv)
 {
 #if defined(HAVE_SETLOCALE) && defined(HAVE_LOCALE_H)
 	setlocale(LC_ALL, "" );
 #endif
-		
+
 	parse_command_line(argc, argv);
-	
+
 	init_abook();
 
 	get_commands();	
-	
+
 	quit_abook();
 
 	return 0;
 }
 
+static void
+free_filenames()
+{
+	my_free(rcfile);
+	my_free(datafile);
+}
+
+
 static void
 set_filenames()
 {
@@ -125,67 +194,178 @@ set_filenames()
 		exit(1);
 	}
 
-	if (!datafile)
-		datafile = strconcat(getenv("HOME"), "/" DATAFILE, NULL);
+	if(!datafile)
+		datafile = strconcat(getenv("HOME"), "/" DIR_IN_HOME "/"
+				DATAFILE, NULL);
 
-	rcfile = strconcat(getenv("HOME"), "/" RCFILE, NULL);
+	if(!rcfile)
+		rcfile = strconcat(getenv("HOME"), "/" DIR_IN_HOME "/"
+				RCFILE, NULL);
 
 	atexit(free_filenames);
 }
 
+/*
+ * CLI
+ */
+
+enum {
+	MODE_CONT,
+	MODE_ADD_EMAIL,
+	MODE_ADD_EMAIL_QUIET,
+	MODE_QUERY,
+	MODE_CONVERT
+};
+
 static void
-free_filenames()
+change_mode(int *current, int mode)
 {
-	my_free(rcfile);
-	my_free(datafile);
+	if(*current != MODE_CONT) {
+		fprintf(stderr, "Cannot combine options --mutt-query, "
+				"--convert, "
+				"--add-email or --add-email-quiet\n");
+		exit(1);
+	}
+
+	*current = mode;
 }
 
+void
+set_filename(char **var, char *path)
+{
+	char *cwd;
+
+	assert(var != NULL);
+	assert(*var == NULL); /* or else we probably leak memory */
+	assert(path != NULL);
+
+	if(*path == '/') {
+		*var = strdup(path);
+		return;
+	}
+
+	cwd = my_getcwd();
+
+	*var = strconcat(cwd, "/", path, NULL);
+
+	free(cwd);
+}
+
+#define set_convert_var(X) do { if(mode != MODE_CONVERT) {\
+	fprintf(stderr, "please use option --%s after --convert option\n",\
+			long_options[option_index].name);\
+		exit(1);\
+	} else\
+		X = optarg;\
+	} while(0)
+
 static void
 parse_command_line(int argc, char **argv)
 {
-	int i;
+	int mode = MODE_CONT;
+	char *query_string = NULL;
+	char *informat = "abook",
+		*outformat = "text",
+		*infile = "-",
+		*outfile = "-";
+	int c;
 
-	for( i = 1; i < argc; i++ ) {
-		if( !strcmp(argv[i], "--help") ) {
-			show_usage();
-			exit(1);
-		} else if( !strcmp(argv[i], "--mutt-query") ) {
-			mutt_query(argv[i + 1]);
-		} else if( !strcmp(argv[i], "--datafile") ) {
-			if (argc > i + 1 ) {
-				if (argv[i+1][0] != '/') {
-					char *cwd = my_getcwd();
-					datafile = strconcat(cwd, "/", argv[i+1], NULL);
-					free(cwd);
-				} else {
-					datafile = strdup(argv[i+1]);
-				}
-				i++;
-			} else {
+	for(;;) {
+		int option_index = 0;
+		enum {
+			OPT_ADD_EMAIL,
+			OPT_ADD_EMAIL_QUIET,
+			OPT_MUTT_QUERY,
+			OPT_CONVERT,
+			OPT_INFORMAT,
+			OPT_OUTFORMAT,
+			OPT_INFILE,
+			OPT_OUTFILE,
+			OPT_FORMATS
+		};
+		static struct option long_options[] = {
+			{ "help", 0, 0, 'h' },
+			{ "add-email", 0, 0, OPT_ADD_EMAIL },
+			{ "add-email-quiet", 0, 0, OPT_ADD_EMAIL_QUIET },
+			{ "datafile", 1, 0, 'f' },
+			{ "mutt-query", 1, 0, OPT_MUTT_QUERY },
+			{ "config", 1, 0, 'C' },
+			{ "convert", 0, 0, OPT_CONVERT },
+			{ "informat", 1, 0, OPT_INFORMAT },
+			{ "outformat", 1, 0, OPT_OUTFORMAT },
+			{ "infile", 1, 0, OPT_INFILE },
+			{ "outfile", 1, 0, OPT_OUTFILE },
+			{ "formats", 0, 0, OPT_FORMATS },
+			{ 0, 0, 0, 0 }
+		};
+
+		c = getopt_long(argc, argv, "hC:",
+				long_options, &option_index);
+
+		if(c == -1)
+			break;
+
+		switch(c) {
+			case 'h':
 				show_usage();
 				exit(1);
-			}
-		} else if( !strcmp(argv[i], "--convert") ) {
-			if( argc < 5 || argc > 6 ) {
-				fprintf(stderr, "incorrect number of argumets to make conversion\n");
-				fprintf(stderr, "try %s --help\n", argv[0]);
+			case OPT_ADD_EMAIL:
+				change_mode(&mode, MODE_ADD_EMAIL);
+				break;
+			case OPT_ADD_EMAIL_QUIET:
+				change_mode(&mode, MODE_ADD_EMAIL_QUIET);
+				break;
+			case 'f':
+				set_filename(&datafile, optarg);
+				alternative_datafile = TRUE;
+				break;
+			case OPT_MUTT_QUERY:
+				query_string = optarg;
+				change_mode(&mode, MODE_QUERY);
+				break;
+			case 'C':
+				set_filename(&rcfile, optarg);
+				alternative_rcfile = TRUE;
+				break;
+			case OPT_CONVERT:
+				change_mode(&mode, MODE_CONVERT);
+				break;
+			case OPT_INFORMAT:
+				set_convert_var(informat);
+				break;
+			case OPT_OUTFORMAT:
+				set_convert_var(outformat);
+				break;
+			case OPT_INFILE:
+				set_convert_var(infile);
+				break;
+			case OPT_OUTFILE:
+				set_convert_var(outfile);
+				break;
+			case OPT_FORMATS:
+				print_filters();
+				exit(0);
+			default:
 				exit(1);
-			}
-			if( argc > i + 4 )
-				convert(argv[i+1], argv[i+2],
-					argv[i+3], argv[i+4]);
-			else
-				convert(argv[i+1], argv[i+2], argv[i+3], "-");
-		} else if( !strcmp(argv[i], "--add-email") ) {
-			add_email(0);
-		} else if( !strcmp(argv[i], "--add-email-quiet") ) {
-			add_email(1);
-		} else {
-			printf("option %s not recognized\n", argv[i]);
-			printf("try %s --help\n", argv[0]);
-			exit(1);
 		}
 	}
+
+	if (optind < argc) {
+		fprintf(stderr, "%s: unrecognized arguments on command line\n",
+				argv[0]);
+		exit(1);
+	}
+
+	switch(mode) {
+		case MODE_ADD_EMAIL:
+			add_email(0);
+		case MODE_ADD_EMAIL_QUIET:
+			add_email(1);
+		case MODE_QUERY:
+			mutt_query(query_string);
+		case MODE_CONVERT:
+			convert(informat, infile, outformat, outfile);
+	}
 }
 
 
@@ -193,11 +373,10 @@ static void
 show_usage()
 {
 	puts	(PACKAGE " v " VERSION "\n");
-	puts	("	--help				show usage");
-	puts	("	--datafile	<filename>	use an alternative addressbook file");
+	puts	("     -h	--help				show usage");
+	puts	("     -C	--config	<file>		use an alternative configuration file");   
+	puts	("	--datafile	<file>		use an alternative addressbook file");
 	puts	("	--mutt-query	<string>	make a query for mutt");
-	puts	("	--convert	<inputformat> <inputfile> "
-		"<outputformat> <outputfile>");
 	puts	("	--add-email			"
 			"read an e-mail message from stdin and\n"
 		"					"
@@ -206,15 +385,34 @@ show_usage()
 		"same as --add-email but doesn't\n"
 		"					confirm adding");
 	putchar('\n');
-	puts	("available formats for --convert option:");
-	print_filters();
-#ifdef DEBUG
-	puts	("\nWarning: this version compiled with DEBUG flag ON");
-#endif
+	puts	("	--convert			convert address book files");
+	puts	("	options to use with --convert:");
+	puts	("	--informat	<format>	format for input file");
+	puts	("					(default: abook)");
+	puts	("	--infile	<file>		source file");
+	puts	("					(default: stdin)");
+	puts	("	--outformat	<format>	format for output file");
+	puts	("					(default: text)");
+	puts	("	--outfile	<file>		destination file");
+	puts	("					(default: stdout)");
+	puts	("	--formats			list available formats");
 }
 
+/*
+ * end of CLI
+ */
+
 extern list_item *database;
 
+static void
+quit_mutt_query(int status)
+{
+	close_database();
+	free_opts();
+
+	exit(status);
+}
+
 static void
 muttq_print_item(FILE *file, int item)
 {
@@ -222,8 +420,8 @@ muttq_print_item(FILE *file, int item)
 	int i;
 
 	split_emailstr(item, emails);
-	
-	for(i = 0; i < (options_get_int("mutt_return_all_emails") ?
+
+	for(i = 0; i < (opt_get_bool(BOOL_MUTT_RETURN_ALL_EMAILS) ?
 			MAX_EMAILS : 1) ; i++)
 		if( *emails[i] )
 			fprintf(file, "%s\t%s\t%s\n", emails[i],
@@ -264,8 +462,9 @@ static void
 init_mutt_query()
 {
 	set_filenames();
-	init_options();
-	
+	init_opts();
+	load_opts(rcfile);
+
 	if( load_database(datafile) ) {
 		printf("Cannot open database\n");
 		quit_mutt_query(1);
@@ -273,15 +472,6 @@ init_mutt_query()
 	}
 }
 
-static void
-quit_mutt_query(int status)
-{
-	close_database();
-	close_config();
-
-	exit(status);
-}
-
 
 static char *
 make_mailstr(int item)
@@ -297,7 +487,7 @@ make_mailstr(int item)
 		strdup(name);
 
 	free(name);
-	
+
 	return ret;
 }
 
@@ -321,7 +511,7 @@ void
 launch_mutt(int item)
 {
 	char *cmd = NULL, *mailstr = NULL;
-	char *mutt_command = options_get_str("mutt_command");
+	char *mutt_command = opt_get_str(STR_MUTT_COMMAND);
 
 	if(mutt_command == NULL || !*mutt_command)
 		return;
@@ -348,7 +538,7 @@ launch_mutt(int item)
 #endif
 	system(cmd);	
 	free(cmd);
-	
+
 	/*
 	 * we need to make sure that curses settings are correct
 	 */
@@ -365,7 +555,7 @@ launch_wwwbrowser(int item)
 
 	if( database[item][URL] )
 		cmd = mkstr("%s '%s'",
-				options_get_str("www_command"),
+				opt_get_str(STR_WWW_COMMAND),
 				safe_str(database[item][URL]));
 	else
 		return;
@@ -419,19 +609,17 @@ abook_fopen (const char *path, const char *mode)
 {	
 	struct stat s;
 	bool stat_ok;
-	
-	stat_ok = (stat(path, &s) != -1);
 
+	stat_ok = (stat(path, &s) != -1);
+	
 	if(strchr(mode, 'r'))
 		return (stat_ok && S_ISREG(s.st_mode)) ?
-				fopen(path, mode) : NULL;
+			fopen(path, mode) : NULL;
 	else
 		return (stat_ok && S_ISDIR(s.st_mode)) ?
 			NULL : fopen(path, mode);
 }
 
-
-
 static void
 convert(char *srcformat, char *srcfile, char *dstformat, char *dstfile)
 {
@@ -442,11 +630,8 @@ convert(char *srcformat, char *srcfile, char *dstformat, char *dstfile)
 		fprintf(stderr, "try --help\n");
 	}
 
-	strlower(srcformat);
-	strlower(dstformat);
-
 #ifndef DEBUG
-	if( !strcmp(srcformat, dstformat) ) {
+	if( !strcasecmp(srcformat, dstformat) ) {
 		printf(	"input and output formats are the same\n"
 			"exiting...\n");
 		exit(1);
@@ -454,9 +639,10 @@ convert(char *srcformat, char *srcfile, char *dstformat, char *dstfile)
 #endif
 
 	set_filenames();
-	init_options();
+	init_opts();
+	load_opts(rcfile);
 
-	switch( import(srcformat, srcfile) ) {
+	switch( import_file(srcformat, srcfile) ) {
 		case -1:
 			fprintf(stderr,
 				"input format %s not supported\n", srcformat);
@@ -469,7 +655,7 @@ convert(char *srcformat, char *srcfile, char *dstformat, char *dstfile)
 	}
 
 	if(!ret)
-		switch( export(dstformat, dstfile) ) {
+		switch( export_file(dstformat, dstfile) ) {
 			case -1:
 				fprintf(stderr,
 					"output format %s not supported\n",
@@ -484,7 +670,7 @@ convert(char *srcformat, char *srcfile, char *dstformat, char *dstfile)
 		}
 
 	close_database();
-	close_config();
+	free_opts();
 	exit(ret);
 }
 
@@ -521,9 +707,11 @@ init_add_email()
 {
 	set_filenames();
 	atexit(free_filenames);
-	init_options();
-	atexit(close_config);
-	
+	check_abook_directory();
+	init_opts();
+	load_opts(rcfile);
+	atexit(free_opts);
+
 	/*
 	 * we don't actually care if loading fails or not
 	 */
@@ -539,6 +727,16 @@ add_email_add_item(int quiet, char *name, char *email)
 {
 	list_item item;
 
+	if(opt_get_bool(BOOL_ADD_EMAIL_PREVENT_DUPLICATES)) {
+		int search_fields[] = { EMAIL, -1 };
+		if(find_item(email, 0, search_fields) >= 0) {
+			if(!quiet)
+				printf("Address %s already in addressbook\n",
+						email);
+			return 0;
+		}
+	}
+					
 	if(!quiet) {
 		FILE *in = fopen("/dev/tty", "r");
 		char c;
@@ -575,7 +773,7 @@ add_email(int quiet)
 {
 	char *line;
 	char *name = NULL, *email = NULL;
-	struct stat s;	
+	struct stat s;
 
 	if( (fstat(fileno(stdin), &s)) == -1 || S_ISDIR(s.st_mode)) {
 		fprintf(stderr, "stdin is a directory or cannot stat stdin\n");
