@@ -23,7 +23,10 @@ int		strncasecmp (const char *, const char *, size_t);
 #define MIN_COLS	70	
 
 #define DEFAULT_UMASK	066
-#define DATAFILE	".abook.addressbook"
+#define DIR_IN_HOME	".abook"
+#define DATAFILE	"addressbook"
+
+#define RCFILE		"abookrc"
 
 /*
  * some "abookwide" useful macros
