@@ -30,14 +30,14 @@ enum {
 };
 
 #define LIST_TOP        3
-#define LIST_BOTTOM     (LINES-3)
+#define LIST_BOTTOM     (LINES-2)
 
 #define LIST_LINES	(LIST_BOTTOM-LIST_TOP)
 #define LIST_COLS	COLS
 
 #define NAMEPOS		2
-#define EMAILPOS        options_get_int("emailpos")
-#define EXTRAPOS	options_get_int("extrapos")
+#define EMAILPOS        opt_get_int(INT_EMAILPOS)
+#define EXTRAPOS	opt_get_int(INT_EXTRAPOS)
 
 #define NAMELEN		(EMAILPOS-NAMEPOS -1)
 #define EMAILLEN        (EXTRAPOS-EMAILPOS -1)
