@@ -1,6 +1,6 @@
 
 /*
- * $Id: misc.c,v 1.7 2001/12/19 20:20:55 jheinonen Exp $
+ * $Id: misc.c,v 1.9 2002/03/18 10:30:14 jheinonen Exp $
  *
  * by JH <jheinonen@users.sourceforge.net>
  *
@@ -32,38 +32,6 @@
 
 #include <assert.h>
 
-char *
-revstr(char *str)
-{
-	char *s, *s2;
-
-	assert(str != NULL);
-
-	s = s2 = strdup(str);
-
-	while( *str )
-		str++;
-
-	while( *s )
-		*--str = *s++;
-
-	free(s2);
-	return str;
-}
-
-char *
-strupper(char *str)
-{
-	char *tmp = str;
-
-	assert(str != NULL);
-
-	while( ( *str = toupper( *str ) ) )
-		str++;
-	
-	return tmp;
-}
-
 char *
 strlower(char *str)
 {
@@ -220,11 +188,11 @@ my_getcwd()
 	char *dir = NULL;
 	int size = 100;
 
-	if( (dir = malloc(size)) == NULL)
+	if( (dir = (char *)malloc(size)) == NULL)
 		return NULL;
 	
 	while( getcwd(dir, size) == NULL && errno == ERANGE )
-		if( (dir = realloc(dir, size *=2)) == NULL)
+		if( (dir = (char *)realloc(dir, size *=2)) == NULL)
 			return NULL;
 
 	return dir;
@@ -266,7 +234,8 @@ getaline(FILE *f)
 			break;		/* the whole line has been read */
 
 		for (inc = size, p = NULL; inc > mininc; inc /= 2)
-			if ((p = abook_realloc(buf, size + inc)) != NULL)
+			if ((p = (char *)abook_realloc(buf, size + inc)) !=
+					NULL)
 				break;
 
 		size += inc;
@@ -282,7 +251,7 @@ getaline(FILE *f)
 		buf[--len] = '\0';
 
 	if (size - len > mucho) { /* a plenitude of unused memory? */
-		p = abook_realloc(buf, len+1);
+		p = (char *)abook_realloc(buf, len+1);
 		if (p != NULL) {
 			buf = p;
 			size = len+1;
