@@ -1,6 +1,6 @@
 
 /*
- * $Id: ui.c,v 1.15 2001/12/19 20:20:55 jheinonen Exp $
+ * $Id: ui.c,v 1.26 2003/05/30 16:24:20 jheinonen Exp $
  *
  * by JH <jheinonen@users.sourceforge.net>
  *
@@ -22,7 +22,6 @@
 #include "misc.h"
 #include "options.h"
 #include "filter.h"
-#include "estr.h"
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
 #endif
@@ -37,6 +36,8 @@
 #	include <sys/ioctl.h>
 #endif
 
+#include "abook_rl.h"
+
 /*
  * external variables
  */
@@ -44,24 +45,25 @@
 extern int items, curitem;
 extern char *datafile;
 
+extern int alternative_datafile;
+
 /*
  * internal variables
  */
 
-int ui_initialized = FALSE;
+bool ui_initialized = FALSE;
 
-int should_resize = FALSE;
-int can_resize = FALSE;
+bool should_resize = FALSE;
+bool can_resize = FALSE;
 
 WINDOW *top = NULL, *bottom = NULL;
 
 
-
 static void
 init_windows()
 {
 	top = newwin(LIST_TOP - 1, COLS, 0, 0);
-	
+
 	bottom = newwin(LINES - LIST_BOTTOM, COLS, LIST_BOTTOM, 0);
 }
 
@@ -182,15 +184,15 @@ void
 headerline(char *str)
 {
 	werase(top);
-	
+
 	mvwhline(top, 1, 0, UI_HLINE_CHAR, COLS);
-	
+
 	mvwprintw(top, 0, 0, "%s | %s", PACKAGE " " VERSION, str);
 
 	refresh();
 	wrefresh(top);
 }
-		
+
 
 void
 refresh_screen()
@@ -202,7 +204,7 @@ refresh_screen()
 	}
 #endif
 	clear();
-	
+
 	refresh_statusline();
 	headerline(MAIN_HELPLINE);
 	list_headerline();
@@ -211,16 +213,20 @@ refresh_screen()
 }
 
 
-void
+int
 statusline_msg(char *msg)
 {
+	int c;
+	
 	clear_statusline();
 	statusline_addstr(msg);
-	getch();
+	c = getch();
 #ifdef DEBUG
 	fprintf(stderr, "statusline_msg(\"%s\")\n", msg);
 #endif
 	clear_statusline();
+
+	return c;
 }
 
 void
@@ -231,53 +237,22 @@ statusline_addstr(char *str)
 	wrefresh(bottom);
 }
 
-/*
- * function statusline_getnstr
- *
- * parameters:
- *  (char *str)
- *   if n >= 0 str is a pointer which points a place where to store
- *   the string, else str is ignored
- *  (int n)
- *   the maximum length of the string
- *   If n < 0 function will allocate needed space for the string.
- *   Value 0 is not allowed for n.
- *  (int use_filesel)
- *   if this value is nonzero the fileselector is enabled
- *
- *  returns (char *)
- *   If n < 0 a pointer to a newly allocated string is returned.
- *   If n > 0 a nonzero value is returned if user has typed a valid
- *   string. If not NULL value is returned. Never really use the
- *   _pointer_ if n > 0.
- *
- */
-
 char *
-statusline_getnstr(char *str, int n, int use_filesel)
+ui_readline(char *prompt, char *s, int limit, bool use_completion)
 {
-	char *buf;
 	int y, x;
+	char *ret;
+
+	mvwaddstr(bottom, 1, 0, prompt);
 
 	getyx(bottom, y, x);
-	wmove(bottom, 1, x);
-	
-	buf = wenter_string(bottom, n,
-			(use_filesel ? ESTR_USE_FILESEL:0) | ESTR_DONT_WRAP);
 
-	if(n < 0)
-		return buf;
-	
-	if(buf == NULL)
-		str[0] = 0;
-	else
-		strncpy(str, buf, n);
+	ret = abook_readline(bottom, y, x, s, limit, use_completion);
 
-	str[n-1] = 0;
+	if(ret)
+		strtrim(ret);
 
-	free(buf);
-
-	return buf;
+	return ret;
 }
 
 int
@@ -285,7 +260,7 @@ statusline_ask_boolean(char *msg, int def)
 {
 	int ret;
 	char *msg2 = strconcat(msg,  def ? " (Y/n)?" : " (y/N)?", NULL);
-			
+
 	statusline_addstr(msg2);
 
 	free(msg2);
@@ -314,24 +289,20 @@ refresh_statusline()
 	werase(bottom);
 
 	mvwhline(bottom, 0, 0, UI_HLINE_CHAR, COLS);
-	mvwhline(bottom, 2, 0, UI_HLINE_CHAR, COLS);
 
 	refresh();
 	wrefresh(bottom);
 }
-	
+
 
 char *
-ask_filename(char *prompt, int flags)
+ask_filename(char *prompt)
 {
 	char *buf = NULL;
 
 	clear_statusline();
-	
-	statusline_addstr(prompt);
-	buf = statusline_getnstr(NULL, -1, flags);
 
-	clear_statusline();
+	buf = ui_readline(prompt, NULL, -1, 1);
 
 	return buf;
 }
@@ -371,7 +342,7 @@ display_help(int help)
 	helpw = newwin(LINES - 5, COLS - 6, 2, 3);
 	erase();
 	headerline("help");
-	
+
 	for( i = 0; tbl[i] != NULL; i++) {
 		waddstr(helpw, tbl[i]);
 		if( ( !( (i+1) % (LINES-8) ) ) ||
@@ -379,7 +350,9 @@ display_help(int help)
 			refresh();
 			wrefresh(helpw);
 			refresh_statusline();
-			statusline_msg("Press any key to continue...");
+			if(statusline_msg("Press any key to continue...")
+					== 'q')
+				break;
 			wclear(helpw);
 		}
 	}
@@ -392,7 +365,7 @@ display_help(int help)
  * end of help
  */
 
-char *selected;
+extern char *selected;
 extern int curitem;
 
 void
@@ -521,10 +494,11 @@ ui_find(int next)
 		if( !*findstr )
 			return;
 	} else {
+		char *s;
 		clear_statusline();
-		statusline_addstr("/");
-		statusline_getnstr(findstr, MAX_FIELD_LEN - 1, 0);
-		clear_statusline();
+		s = ui_readline("/", findstr, MAX_FIELD_LEN - 1, 0);
+		strncpy(findstr, s, MAX_FIELD_LEN);
+		refresh_screen();
 	}
 
 	if( (item = find_item(findstr, curitem + !!next,
@@ -563,7 +537,7 @@ void
 ui_print_database()
 {
 	FILE *handle;
-	char *command = options_get_str("print_command");
+	char *command = opt_get_str(STR_PRINT_COMMAND);
 	int mode;
 
 	if( list_is_empty() )
@@ -603,14 +577,15 @@ ui_open_datafile()
 {
 	char *filename;
 
-	filename = ask_filename("File to open: ", 1);
+	filename = ask_filename("File to open: ");
 
-	if( !filename ) {
+	if( !filename || ! *filename) {
+		free(filename);
 		refresh_screen();
 		return;
 	}
 
-	if( options_get_int("autosave") )
+	if( opt_get_bool(BOOL_AUTOSAVE) )
 		save_database();
 	else if(statusline_ask_boolean("Save current database", FALSE))
 		save_database();
@@ -629,4 +604,6 @@ ui_open_datafile()
 
 	refresh_screen();
 	free(filename);
+
+	alternative_datafile = TRUE;
 }
